@@ -343,6 +343,30 @@ void V4PCI_DSP::SetHookState( int port, HookState hookstate )
 	}
 } //}}}
 
+static uint16_t silence[V4PCI_LSF];
+
+void V4PCI_DSP::PadTxFrame( int port )
+{ //{{{
+	size_t remains = m_reg.txdf[port]->HowFull() % V4PCI_LSF;
+
+	if(remains)
+		m_reg.txdf[port]->Write(silence, V4PCI_LSF - remains);
+} //}}}
+
+void V4PCI_DSP::WaitForTxEmpty( int port )
+{ //{{{
+	size_t remains;
+
+	while((remains = m_reg.txdf[port]->HowFull()) >= V4PCI_LSF)
+		usleep( 10 * 1000 );
+
+	if( remains ) {
+		m_reg.txdf[port]->Write(silence, V4PCI_LSF - remains);
+		usleep( 20 * 1000 );
+	}
+} //}}}
+
+
 void V4PCI_DSP::UNListen( unsigned int port )
 { //{{{
 	mprintf("[%d/%d] V4PCI_DSP::UNListen: not implemented\n",
