@@ -33,7 +33,14 @@
 #include <errno.h>
 #include <pthread.h>
 
-#include "sangoma.h"
+#include <cstdlib>
+#include <cstring>
+#include <sys/ioctl.h>
+#include <sys/types.h>
+#include <sys/socket.h>
+#include <arpa/inet.h>
+#include <linux/if_wanpipe.h>
+
 #include "openpri.h"
 #include "hostvox.h"
 #include "mess.h"
@@ -48,6 +55,12 @@ extern "C" {
 #include "toned.h"
 #include "libpri/pri_q931.h"
 #include "libpri/pri_q921.h"
+
+// This one we include to get at the q931_call struct for slotmap support.
+// Don't make a habit of using things from here, anything that is needed
+// really should be made part of the public interface instead.
+// ... but for now, leave libpri internals alone.
+#include "libpri/pri_internal.h"
 }
 
 
@@ -217,6 +230,9 @@ void OpenPri::HostDSPOpen()
 		chans[i].v = &m_reg;
 		chans[i].call = NULL;
 		chans[i].tx_fifo = NULL;
+		chans[i].linear_play_gain = pow(10.0, m_reg.defPlayGain / 20.0);
+		chans[i].linear_record_gain = pow(10.0, m_reg.defRecordGain / 20.0);
+
 //		mprintf("Started channel [%d] on board [%d] with handle [%d] indexed [%d]\n",
 //			chans[i].chan_num, m_reg.cardnum, chans[i].hand_num, i);
 //		mprintf("[%d/%d] Started channel with handle [%d] indexed [%d]\n",
@@ -365,25 +381,21 @@ void OpenPri::ClosePorts()
 
 // This emulates the hardware gain as per the other cards
 // (and is a clone of the software gain_vector function from playrec.cpp)
-// XXX Does this mean we really do software gain _twice_ for PRI cards?
-//     That would be dumb -- this touches every byte of audio.
+// This is applied to all signals, unlike the software gain which only is
+// applied to samples processed by vpb_{play,record} functions.
 static inline void hw_gain_vector(float g, short *v, int n)
 { //{{{
 	// This is the maximum scale factor that lets us safely use
 	// an int instead of a float as our temporary computed value.
 	const unsigned int scalemax = INT_MAX / 32768;
 
-	//XXX Compute this elsewhere when the raw gain value is set
-	//    instead of doing it every time this function is called.
-	g = pow(10.0, g/20.0);
-
 	if( g > scalemax ) g = scalemax;
 	for(short *end = v + n; v != end; ++v)
 	{
-		int tmp = (int)(*v * g);
-		if(tmp > 32767)       *v = 32767;
-		else if(tmp < -32768) *v = -32768;
-		else                  *v = tmp;
+		int tmp = lrintf(*v * g);
+		if(__builtin_expect(tmp > 32767,0))       *v = 32767;
+		else if(__builtin_expect(tmp < -32768,0)) *v = -32768;
+		else                                      *v = tmp;
 	}
 } //}}}
 
@@ -422,7 +434,8 @@ void prepare_play_buf(unsigned char alawbuf[NBUF], VPBREG *v, int ch, int encodi
 	if(tonegen) tonegen->MixTone( linbuf, NBUF );
 
 	/* Apply the 'hardware' gain */
-	hw_gain_vector(v->defPlayGain, linbuf, NBUF);
+	hw_gain_vector(chans[idx].linear_play_gain, linbuf, NBUF);
+
 	/* Needs to check if we are alaw or mu-law */
 	if (encoding == VPB_ALAW){
 		alaw_encode((char *)&alawbuf[0], linbuf, NBUF);
@@ -482,7 +495,7 @@ void prepare_rec_buf(unsigned char alawbuf[NBUF], VPBREG *v, int ch, int encodin
 	}
 
 	/* Apply the Hardware gain */
-	hw_gain_vector(v->defRecordGain, linbuf, NBUF);
+	hw_gain_vector(chans[idx].linear_record_gain, linbuf, NBUF);
 
 #ifdef SOFTBRIDGE
 	if (chans[ch].vox_states) {
@@ -621,7 +634,7 @@ void prepare_rec_buf_log(unsigned char alawbuf[NBUF],
 	// In past, gain of 1.12 was used....not worth it??
 	// 
 	/* Apply the Hardware gain */
-	hw_gain_vector(v->defRecordGain, linbuf, NBUF);
+	hw_gain_vector(chans[idx].linear_record_gain, linbuf, NBUF);
     #endif
 
 	/* write it to the buffer */
@@ -865,10 +878,12 @@ bool OpenPri::ProcessCommands()
 
 	    case PC_CODEC_RECGAIN:
 		assert(mess[0] == PC_LCODEC_RECGAIN);
+		chans[idx].linear_record_gain = pow(10.0, ((int)mess[3] - 0x80) / 200.0);
 		break;
 
 	    case PC_CODEC_PLAYGAIN:
 		assert(mess[0] == PC_LCODEC_PLAYGAIN);
+		chans[idx].linear_play_gain = pow(10.0, ((int)mess[3] - 0x80) / 200.0);
 		break;
 
 	  #if 0
@@ -1363,7 +1378,7 @@ void *OpenPri::RunDchannelLog(void *p)
 		// else mprintf("[%ld]No events, checking channel statii...\n",ts.tv_sec);
 
 		for (x=0; x < v->numch; ++x) {
-			//mprintf("Checking [%d] state[%d]\n",x,chans[x].state);
+			// mprintf("Checking [%d] state[%d]\n",x,chans[x].state);
 			switch(chans[x].state){
 			case CH_NEEDHANG:
 				pthread_mutex_lock(&openpri->m_mutex);
@@ -1402,6 +1417,7 @@ void *OpenPri::RunDchannelLog(void *p)
 				break;
 
 			case CH_NEEDANS:
+				pthread_mutex_lock(&openpri->m_mutex);
 				chans[x].run=1;
 				//pthread_create(&audio_thread_t[x], NULL,
 				//		 audio_thread_log,(void*)&chans[x]);
@@ -1431,7 +1447,7 @@ void *OpenPri::RunDchannelLog(void *p)
 									cref_info_last->next = cref_info->next;
 								}
 								else {
-									openpri->m_cref_info = NULL;
+									openpri->m_cref_info = cref_info->next;
 								}
 								delete cref_info;
 								cref_info = NULL;
@@ -1454,9 +1470,11 @@ void *OpenPri::RunDchannelLog(void *p)
 				rmess[1] = DSP_ISDN_ANS;
 				rmess[2] = chans[x].chan_num;
 				v->upmess->Write(rmess, rmess[0]);
+				pthread_mutex_unlock(&openpri->m_mutex);
 				break;
 
 			case CH_NEEDRING:
+				pthread_mutex_lock(&openpri->m_mutex);
 				if (chans[x].cid)
 					mprintf("[%d/%02d] Sending ring event up...(cref:%d)(%s %s %s)\n",
 						cardnum,chans[x].chan_num,
@@ -1466,11 +1484,13 @@ void *OpenPri::RunDchannelLog(void *p)
 					mprintf("[%d/%02d] Sending ring event up...(cref:%d)\n",
 						cardnum,chans[x].chan_num,
 						chans[x].cref);
+
 				rmess[0] = DSP_LISDN_RING;
 				rmess[1] = DSP_ISDN_RING;
 				rmess[2] = chans[x].chan_num;
 				v->upmess->Write(rmess, rmess[0]);
 				chans[x].state=CH_RING;
+				pthread_mutex_unlock(&openpri->m_mutex);
 				break;
 
 			case CH_NEEDTERMINATING:
@@ -1682,9 +1702,14 @@ void OpenPri::handle_pri_event_log(struct pri *pri, pri_event *e)
 		break;
 
 	    case PRI_EVENT_RING:
+		//XXX ring.call is internal libpri state, not public state data
+		if ((e->ring.channel == -1) && (e->ring.call->slotmap > 0))
+			e->ring.channel = ffs(e->ring.call->slotmap);
+
 		mprintf("[%d/%d] PRI Ring on channel (from [%s] to [%s] sub [%s])(cref:%d)\n",
 			m_reg.cardnum, e->ring.channel, e->ring.callingnum,
 			e->ring.callednum,e->ring.calledsubaddr,e->ring.cref);
+
 		if (e->ring.channel != -1) {
 			try {
 				if (idx == -1)
@@ -1707,6 +1732,7 @@ void OpenPri::handle_pri_event_log(struct pri *pri, pri_event *e)
 			pthread_mutex_unlock(&m_mutex);
 		}
 		else {
+			pthread_mutex_lock(&m_mutex);
 			crefinfo = m_cref_info;
 			if(crefinfo != NULL){
 				while(crefinfo->next != NULL)
@@ -1720,6 +1746,7 @@ void OpenPri::handle_pri_event_log(struct pri *pri, pri_event *e)
 			}
 			crefinfo->next = NULL;
 			set_callinfo(*crefinfo, e->ring);
+			pthread_mutex_unlock(&m_mutex);
 		}
 		break;
 
@@ -1738,6 +1765,7 @@ void OpenPri::handle_pri_event_log(struct pri *pri, pri_event *e)
 		// need to ensure that we clean up the crefinfo linked list,
 		// otherwise the cref of unanswered calls remains in the list
 		// and leaks memory
+		pthread_mutex_lock(&m_mutex);
 		crefinfo = m_cref_info;
 		prevcrefinfo = NULL;
 		while (crefinfo && crefinfo->cref != (e->hangup.cref&0x7fff)) {
@@ -1751,6 +1779,7 @@ void OpenPri::handle_pri_event_log(struct pri *pri, pri_event *e)
 				m_cref_info = crefinfo->next;
 			delete crefinfo;
 		}
+		pthread_mutex_unlock(&m_mutex);
 
 		try {
 			if (idx == -1)
@@ -1966,12 +1995,12 @@ void OpenPri::handle_pri_event_log(struct pri *pri, pri_event *e)
 						e->ring.channel , chans[idx].cid->callednum ,e->ring.cref);
 
 			}
-
 			pthread_mutex_unlock(&m_mutex);
 		} else {
 			// no channel number selected yet - need to keep track
 			// of information in cref_info linked list for now
 
+			pthread_mutex_lock(&m_mutex);
 			crefinfo = m_cref_info;
 			while (crefinfo != NULL && crefinfo->cref != (e->ring.cref&0x7fff))
 				crefinfo = crefinfo->next;
@@ -1981,6 +2010,7 @@ void OpenPri::handle_pri_event_log(struct pri *pri, pri_event *e)
 				mprintf("[%d/%d] PRI called number is now : %s (cref:%d)\n",m_reg.cardnum,
 						e->ring.channel, crefinfo->cid.callednum, e->ring.cref);
 			}
+			pthread_mutex_unlock(&m_mutex);
 		}
 		break;
 
@@ -2511,7 +2541,7 @@ int OpenPri::make_isdn_call(uint16_t *mp)
 
 	pthread_mutex_unlock(&m_mutex);
 
-	if (ret ==1 ){
+	if (ret) {
 		mprintf("[%d/%02d] Couldnt set up call!",
 			m_reg.cardnum, chans[idx].chan_num);
 		return 4;
@@ -2589,12 +2619,23 @@ void OpenPri::SetHookState( int port, HookState hookstate )
 	}
 } //}}}
 
+//static uint16_t silence[NBUF];
+
+void OpenPri::PadTxFrame( int port )
+{ //{{{
+//XXX What, if anything, do we need to do here?
+#if 0
+	int remains = m_reg.txdf[port]->HowFull() % NBUF;
+
+	if(remains)
+		m_reg.txdf[port]->Write(silence, NBUF - remains);
+#endif
+} //}}}
+
 void OpenPri::WaitForTxEmpty( int port )
 { //{{{
 //XXX What, if anything, do we need to do here?
 #if 0
-	static uint16_t     silence[NBUF];
-
 	for(;;) {
 		int remains = m_reg.txdf[port]->HowFull();
 
@@ -2892,7 +2933,7 @@ int OpenPri::make_isdn_call_api(int ch, char *number, char *cid, int layer1,
 
 	pthread_mutex_unlock(&m_mutex);
 
-	if (ret ==1 ){
+	if (ret) {
 		mprintf("[%d/%02d] Couldnt set up call!",m_reg.cardnum,chans[idx].chan_num);
 		return 4;
 	}
