@@ -48,6 +48,7 @@
 #include "coff.h"
 #include "vtcore.h"
 #include "alawmulaw.h"
+#include "scopedmutex.h"
 
 #ifndef __FreeBSD__
   #include "loopmon.h"
@@ -69,7 +70,6 @@
 
 #define MESSEVENTDELAY	20		// how often VPBs are polled by driver
 #define APIQUEUESIZE	128		// default max num events in API Q
-#define MASK_EVENTS	9		// number of maskable API events
 
 #define	SIZE_OF_VPB_EVENT_WORDS	(sizeof(VPB_EVENT)/sizeof(uint16_t))
 
@@ -77,15 +77,6 @@ static size_t global_apiq_size  = APIQUEUESIZE * SIZE_OF_VPB_EVENT_WORDS;
 static size_t channel_apiq_size = APIQUEUESIZE * SIZE_OF_VPB_EVENT_WORDS;
 
 
-// Call progress state machine event codes.
-// These codes are passed up in the message from the DSP when a CP tone
-// is detected.
-
-#define	toned_DIAL	0		// dial tone
-#define	toned_RING	1		// ring
-#define	toned_BUSY	2		// busy
-
-
 //! @defgroup VPBapiImpl VPBapi implementation detail
 //! @ingroup InternalImpl
 //! @brief Implementation support found in @c vpbapi.cpp
@@ -184,16 +175,11 @@ static pthread_t mmq_thread;
 
 
 GENERIC_CRITICAL_SECTION	SoftBridgeSect;
-//#define SB_BUFF 160		/* 20 ms of data */
-//#define SB_BUFF 80		/* 10 ms of data */
-#define SB_BUFF 40		/* 5 ms of data */
-//#define SB_BUFF 8		/* 1 ms of data */
 
 // Critical section for put_evt, necessary because the driver via the
 // polling timer may call putevt at the same time as the application.
 
 GENERIC_CRITICAL_SECTION	PutEvtSect;
-GENERIC_CRITICAL_SECTION	VpbOpenSect;
 
 
 // Tone detector params describing Australian/Studio 308/US dial tone 
@@ -819,7 +805,7 @@ void set_country(int board, int port, const Country *country)
 } //}}}
 
 // For internal use, does not validate the handle.
-static void set_country(int handle, const Country *country)
+static void set_country(VPBPortHandle handle, const Country *country)
 { //{{{
 	unsigned short  bd, ch;
 
@@ -836,7 +822,7 @@ const Country *get_country(int board, int port)
 } //}}}
 
 // For internal use, does not validate the handle.
-static const Country *get_country(int handle)
+static const Country *get_country(VPBPortHandle handle)
 { //{{{
 	unsigned short  bd, ch;
 	maphndletodev(handle, &bd, &ch);
@@ -863,19 +849,15 @@ void WINAPI vpb_set_country(const Country *country)
 	}
 } //}}}
 
-int WINAPI vpb_set_country(int handle, const Country *country)
+int WINAPI vpb_set_country(VPBPortHandle handle, const Country *country)
 { //{{{
-	try {
-		ValidHandleCheck(handle);
-		set_country(handle, country);
-	}
-	catch(const Wobbly &w){
-		return RunTimeError(w,"vpb_set_country");
-	}
+	CheckHandle(handle, "vpb_set_country");
+	set_country(handle, country);
+
 	return VPB_OK;
 } //}}}
 
-const Country* WINAPI vpb_get_port_country(int handle)
+const Country* WINAPI vpb_get_port_country(VPBPortHandle handle)
 { //{{{
 	CheckHandle(handle, "vpb_get_port_country");
 	return get_country(handle);
@@ -1513,6 +1495,9 @@ static void ConfigureVPB(Comm *c, unsigned short numboards)
 // internal driver initialisation function
 static void init_vpb()
 { //{{{
+	static pthread_mutex_t	mutex = PTHREAD_MUTEX_INITIALIZER;
+	ScopedMutex		lock(&mutex);
+
 	if(Init) return;
 
 	mess_init();
@@ -1589,13 +1574,9 @@ static void init_vpb()
 	}
       #endif
 
-	int ret = pthread_create(&mmq_thread, NULL,
-				 MonitorMessageQueue, NULL);
-	if( ret ) {
-		mprintf("FAILED to start MMQ thread: %s\n",
-			strerror(ret));
-		throw Wobbly(VPBAPI_NOT_OPEN);
-	}
+	int ret = pthread_create(&mmq_thread, NULL, MonitorMessageQueue, NULL);
+	if( ret )
+		throw VpbException("Failed to start MMQ thread: %s", strerror(ret));
 
 	mprintf("Driver initialised OK\n");
 	Init = 1;
@@ -1609,38 +1590,42 @@ VPBPortHandle WINAPI vpb_open(unsigned int board,
 		init_vpb();
 
 		if(board >= numboards)
-			throw Wobbly(VPBAPI_INVALID_BOARD_NUMBER);
+			throw VpbException("vpb_open: invalid board number %u (of %u)",
+					   board, numboards);
 
 		switch( vpb_c->vpbreg(board)->model )
 		{
 		    case VPB_V4LOG:
 		    case VPB_V4PCI:
 			if(port >= 4)
-				throw Wobbly(VPBAPI_INVALID_CHANNEL_NUMBER);
+				throw VpbException("vpb_open: invalid port number %u (of 4)", port);
 			break;
 
 		    case VPB_OSW:
 			if(port >= 12)
-				throw Wobbly(VPBAPI_INVALID_CHANNEL_NUMBER);
+				throw VpbException("vpb_open: invalid port number %u (of 12)", port);
 			break;
 
 		    case VPB_OPCI:
 			if(port >= vpb_c->vpbreg(board)->numch)
-				throw Wobbly(VPBAPI_INVALID_CHANNEL_NUMBER);
+				throw VpbException("vpb_open: invalid port number %u (of %u)",
+						   port, vpb_c->vpbreg(board)->numch);
 			break;
 
 		    case VPB_PRI:
 			if(port >= 32)
-				throw Wobbly(VPBAPI_INVALID_CHANNEL_NUMBER);
+				throw VpbException("vpb_open: invalid port number %u (of 32)", port);
 			break;
 
 		    case VPB_MODEL_UNKNOWN:
-			throw VpbException("vpb_open: invalid model type");
+			throw VpbException("vpb_open: invalid model type %u",
+					   vpb_c->vpbreg(board)->model);
 		}
 
 		VPBPortHandle h = mapdevtohndle(board, port);
 		if (vpb_dev[h].devstate == VPB_ON)
-			throw Wobbly(VPBAPI_CHANNEL_ALREADY_OPEN);
+			throw VpbException("vpb_open: board %u, port %u, is already open",
+					   board, port);
 
 		vpb_dev[h].devstate = VPB_ON;
 
@@ -1669,6 +1654,17 @@ void WINAPI vpb_reset(VPBPortHandle handle)
 	unsigned short  b, ch;
 	maphndletodev(handle, &b, &ch);
 
+	vpb_tone_terminate(handle);
+	vpb_play_terminate(handle);
+	vpb_record_terminate(handle);
+
+	// This one will throw if it wasn't active, ignore that here.
+	try { vpb_log_terminate(handle); } catch(...) {}
+	vpb_monitor_terminate(handle);
+
+	vpb_flush_events(handle);
+	vpb_flush_digits(handle);
+
 	vpb_c->vpbreg(b)->hostdsp->Reset(ch);
 
 	switch( vpb_get_port_type(handle) )
@@ -1746,7 +1742,7 @@ int WINAPI vpb_close(VPBPortHandle handle)
 		if( handle >= 0 ) {
 			mprintf("vpb_close: handle %d\n", handle);
 
-			ValidHandleCheck(handle);
+			CheckHandle( handle, "vpb_close" );
 			close_port(handle);
 
 			for(int i=0; i < Totalchans; ++i)
@@ -1816,182 +1812,86 @@ int WINAPI vpb_close(VPBPortHandle handle)
 	return VPB_OK;
 } //}}}
 
-int WINAPI vpb_set_event_callback(int handle,
+int WINAPI vpb_set_event_callback(VPBPortHandle handle,
 				  vpb_event_callback callback,
 				  void *context)
 { //{{{
-	try {
-		ValidHandleCheck(handle);
+	CheckHandle( handle, "vpb_set_event_callback" );
 
-		VPB_DEV &dev = vpb_dev[handle];
+	VPB_DEV &dev = vpb_dev[handle];
+
+	pthread_mutex_lock( &dev.callback_mutex );
+	dev.event_callback = callback;
+	dev.context        = context;
+	pthread_mutex_unlock( &dev.callback_mutex );
 
-		pthread_mutex_lock( &dev.callback_mutex );
-		dev.event_callback = callback;
-		dev.context        = context;
-		pthread_mutex_unlock( &dev.callback_mutex );
-	}
-	catch(const Wobbly &w){
-		return RunTimeError(w,"vpb_set_event_callback");
-	}
 	return VPB_OK;
 } //}}}
 
-int WINAPI vpb_set_event_mask(int handle, unsigned long mask)
+int WINAPI vpb_set_event_mask(VPBPortHandle handle, unsigned long mask)
 { //{{{
-	try {
-		ValidHandleCheck(handle);
-		vpb_dev[handle].evtmask = mask;
-	}
-	catch(const Wobbly &w){
-		return RunTimeError(w,"vpb_set_event_mask");
-	}
+	CheckHandle( handle, "vpb_set_event_mask" );
+	vpb_dev[handle].evtmask = mask;
+
 	return VPB_OK;
 } //}}}
 
-unsigned long WINAPI vpb_get_event_mask(int handle)
+unsigned long WINAPI vpb_get_event_mask(VPBPortHandle handle)
 { //{{{
 	CheckHandle(handle, "vpb_get_event_mask");
 	return vpb_dev[handle].evtmask;
 } //}}}
 
-/*---------------------------------------------------------------------------*\
-
-	FUNCTION: vpb_enable_event
-	AUTHOR..: David Rowe
-	DATE....: 7/2/98
-
-	Function to unmask (enable) only the specified events while leaving the
-	rest of the event masks unchanged.
-
-	int		handle	device handle
-	unsigned long	mask	event(s) to unmask		
-
-\*--------------------------------------------------------------------------*/
-
-int WINAPI vpb_enable_event(int handle, unsigned long mask)
+int WINAPI vpb_enable_event(VPBPortHandle handle, unsigned long mask)
 { //{{{
-	try {
-		ValidHandleCheck(handle);
-		vpb_dev[handle].evtmask |= mask;
-	}
-	catch(const Wobbly &w){
-		return RunTimeError(w,"vpb_enable_event");
-	}
+	CheckHandle(handle, "vpb_enable_event");
+	vpb_dev[handle].evtmask |= mask;
+
 	return VPB_OK;
 } //}}}
 
-/*---------------------------------------------------------------------------*\
+int WINAPI vpb_disable_event(VPBPortHandle handle, unsigned long mask)
+{ //{{{
+	CheckHandle(handle, "vpb_disable_event");
+	vpb_dev[handle].evtmask &= ~mask;
 
-	FUNCTION: vpb_disable_event
-	AUTHOR..: David Rowe
-	DATE....: 7/2/98
-
-	Function to mask out (disable) specified events only, while leaving the
-	rest of the event masks unchanged.
-
-\*--------------------------------------------------------------------------*/
-
-int WINAPI vpb_disable_event(int handle, unsigned long mask)
-//	int				handle	device handle
-//	unsigned long	mask	evt(s) to mask
-{
-	try {
-		ValidHandleCheck(handle);
-		vpb_dev[handle].evtmask &= ~mask;
-	}
-	catch(const Wobbly &w){
-		return RunTimeError(w,"vpb_disable_event");
-	}
 	return VPB_OK;
-}
+} //}}}
 
-/*---------------------------------------------------------------------------*\
+int WINAPI vpb_set_tone_event_mask(VPBPortHandle handle, unsigned long mask)
+{ //{{{
+	CheckHandle(handle, "vpb_set_tone_event_mask");
+	vpb_dev[handle].tonemask = mask;
 
-	FUNCTION: vpb_set_tone_event_mask
-	AUTHOR..: John Kostogiannis - Ben Kramer
-	DATE....: 4/12/97 - 30/08/05
-
-	Function to set the TONE event mask for each device.  Each call to this
-	function overwrites the previous event mask.
-
-\*--------------------------------------------------------------------------*/
-
-int WINAPI vpb_set_tone_event_mask(int handle, unsigned long mask)
-//     int                             handle  device handle
-//     unsigned long   mask    new mask to assign to this device
-{
-	try {
-		ValidHandleCheck(handle);
-		vpb_dev[handle].tonemask = mask;
-	}
-	catch(const Wobbly &w){
-		return RunTimeError(w,"vpb_set_tone_event_mask");
-	}
 	return VPB_OK;
-}
+} //}}}
 
-unsigned long WINAPI vpb_get_tone_event_mask(int handle)
+unsigned long WINAPI vpb_get_tone_event_mask(VPBPortHandle handle)
 { //{{{
 	CheckHandle(handle, "vpb_get_tone_event_mask");
 	return vpb_dev[handle].tonemask;
 } //}}}
 
-/*---------------------------------------------------------------------------*\
+int WINAPI vpb_enable_tone_event(VPBPortHandle handle, unsigned long mask)
+{ //{{{
+	CheckHandle(handle, "vpb_enable_tone_event");
+	vpb_dev[handle].tonemask |= mask;
 
-	FUNCTION: vpb_enable_tone_event
-	AUTHOR..: David Rowe - Ben Kramer
-	DATE....: 7/2/98 - 30/08/05
-
-	Function to unmask (enable) only the specified TONE events while leaving the
-	rest of the TONE event masks unchanged.
-
-\*--------------------------------------------------------------------------*/
-
-int WINAPI vpb_enable_tone_event(int handle, unsigned long mask)
-//     int                             handle  device handle
-//     unsigned long   mask    event(s) to unmask
-{
-	try {
-		ValidHandleCheck(handle);
-		vpb_dev[handle].tonemask |= mask;
-	}
-	catch(const Wobbly &w){
-		return RunTimeError(w,"vpb_enable_tone_event");
-	}
 	return VPB_OK;
-}
+} //}}}
 
+int WINAPI vpb_disable_tone_event(VPBPortHandle handle, unsigned long mask)
+{ //{{{
+	CheckHandle(handle, "vpb_disable_tone_event");
+	vpb_dev[handle].tonemask &= ~mask;
 
-/*---------------------------------------------------------------------------*\
-
-	FUNCTION: vpb_disable_tone_event
-	AUTHOR..: David Rowe - Ben Kramer
-	DATE....: 7/2/98 - 30/08/05
-
-	Function to mask out (disable) specified TONE events only, while leaving the
-	rest of the TONE event masks unchanged.
-
-\*--------------------------------------------------------------------------*/
-
-int WINAPI vpb_disable_tone_event(int handle, unsigned long mask)
-//     int                             handle  device handle
-//     unsigned long   mask    evt(s) to mask
-{
-	try {
-		ValidHandleCheck(handle);
-		vpb_dev[handle].tonemask &= ~mask;
-	}
-	catch(const Wobbly &w){
-		return RunTimeError(w,"vpb_disable_tone_event");
-	}
 	return VPB_OK;
-}
-
-
+} //}}}
 
 int WINAPI vpb_put_event(VPB_EVENT *e)
 { //{{{
-	if( ! APIQ ) throw Wobbly(VPBAPI_NOT_OPEN);
+	if( ! APIQ )
+		throw VpbException("vpb_put_event: no global event queue?");
 
 	try {
 		// prevent App and poll timer calling at the same time
@@ -2010,21 +1910,38 @@ int WINAPI vpb_put_event(VPB_EVENT *e)
 	return VPB_OK;
 } //}}}
 
-int WINAPI vpb_get_event_count_ch(int handle)
+int WINAPI vpb_get_event_count_ch(VPBPortHandle handle)
 { //{{{
 	return vpb_dev[handle].APIQ.HowFull() / SIZE_OF_VPB_EVENT_WORDS;
 } //}}}
 
 int WINAPI vpb_get_event_count()
 { //{{{
-	if( ! APIQ ) throw Wobbly(VPBAPI_NOT_OPEN);
+	if( ! APIQ )
+		throw VpbException("vpb_get_event_count: no global event queue?");
+
 	return APIQ->HowFull() / SIZE_OF_VPB_EVENT_WORDS;
 } //}}}
 
+void WINAPI vpb_flush_events(VPBPortHandle handle)
+{ //{{{
+	CheckHandle( handle, "vpb_flush_events" );
+	vpb_dev[handle].APIQ.Flush();
+} //}}}
+
+void WINAPI vpb_flush_events()
+{ //{{{
+	if( ! APIQ )
+		throw VpbException("vpb_flush_events: no global event queue?");
+
+	APIQ->Flush();
+} //}}}
+
 int putevt(VPB_EVENT *e, unsigned long mask)
 { //{{{
 
-	if( ! APIQ ) throw Wobbly(VPBAPI_NOT_OPEN);
+	if( ! APIQ )
+		throw VpbException("putevt: no global event queue?");
 
 	VPB_DEV &dev = vpb_dev[e->handle];
 
@@ -2079,7 +1996,8 @@ int putevt(VPB_EVENT *e, unsigned long mask)
 
 int WINAPI vpb_get_event_async(VPB_EVENT *e)
 { //{{{
-	if( ! APIQ ) throw Wobbly(VPBAPI_NOT_OPEN);
+	if( ! APIQ )
+		throw VpbException("vpb_get_event_async: no global event queue?");
 
 	if( APIQ->Read((unsigned short*)e, SIZE_OF_VPB_EVENT_WORDS) == Fifo::OK )
 		return VPB_OK;
@@ -2092,7 +2010,8 @@ int WINAPI vpb_get_event_sync(VPB_EVENT *e, unsigned int time_out)
 { //{{{
 	unsigned int tick = 0;
 
-	if( ! APIQ ) throw Wobbly(VPBAPI_NOT_OPEN);
+	if( ! APIQ )
+		throw VpbException("vpb_get_event_sync: no global event queue?");
 
 	while( APIQ->Read((unsigned short*)e, SIZE_OF_VPB_EVENT_WORDS) != Fifo::OK )
 	{
@@ -2105,13 +2024,11 @@ int WINAPI vpb_get_event_sync(VPB_EVENT *e, unsigned int time_out)
 	return VPB_OK;
 } //}}}
 
-int WINAPI vpb_get_event_ch_sync(int h, VPB_EVENT *e, unsigned int time_out)
+int WINAPI vpb_get_event_ch_sync(VPBPortHandle h, VPB_EVENT *e, unsigned int time_out)
 { //{{{
-    try {
+	CheckHandle( h, "vpb_get_event_ch_sync" );
+
 	unsigned int tick = 0;
-
-	ValidHandleCheck(h);
-
 	while( vpb_dev[h].APIQ.Read((unsigned short*)e, SIZE_OF_VPB_EVENT_WORDS) != Fifo::OK )
 	{
 		GenericSleep(sleepms);
@@ -2121,25 +2038,16 @@ int WINAPI vpb_get_event_ch_sync(int h, VPB_EVENT *e, unsigned int time_out)
 		}
 	}
 	return VPB_OK;
-    }
-    catch(const Wobbly &w){
-	    return RunTimeError(w,"vpb_get_event_ch_sync");
-    }
 } //}}}
 
-int WINAPI vpb_get_event_ch_async(int h, VPB_EVENT *e)
+int WINAPI vpb_get_event_ch_async(VPBPortHandle h, VPB_EVENT *e)
 { //{{{
-    try {
-	ValidHandleCheck(h);
+	CheckHandle( h, "vpb_get_event_ch_async" );
 	if( vpb_dev[h].APIQ.Read((unsigned short*)e, SIZE_OF_VPB_EVENT_WORDS) == Fifo::OK )
 		return VPB_OK;
 
 	e->type = VPB_EVT_NONE;
 	return VPB_NO_EVENTS;
-    }
-    catch(const Wobbly &w){
-	return RunTimeError(w,"vpb_get_event_ch_aync");
-    }
 } //}}}
 
 void WINAPI vpb_sethook_async(VPBPortHandle handle, HookState hookstate)
@@ -2565,7 +2473,7 @@ VPB_MODEL get_board_type( int board )
 	return vpb_c->vpbreg(board)->model;
 } //}}}
 
-VPB_MODEL WINAPI vpb_get_card_type(int handle)
+VPB_MODEL WINAPI vpb_get_card_type(VPBPortHandle handle)
 { //{{{
 	CheckHandle(handle, "vpb_get_card_type");
 
@@ -2665,7 +2573,7 @@ void get_board_model( int board, char *s )
 	}
 } //}}}
 
-std::string WINAPI vpb_get_model(int handle)
+std::string WINAPI vpb_get_model(VPBPortHandle handle)
 { //{{{
 	CheckHandle(handle, "vpb_get_model");
 
@@ -2678,18 +2586,15 @@ std::string WINAPI vpb_get_model(int handle)
 	return s;
 } //}}}
 
-int WINAPI vpb_get_model(int handle, char *s)
+int WINAPI vpb_get_model(VPBPortHandle handle, char *s)
 { //{{{
-	try {
-		unsigned short bd,ch;
+	CheckHandle(handle, "vpb_get_model");
+
+	unsigned short bd,ch;
+
+	maphndletodev(handle, &bd, &ch);
+	get_board_model(bd, s);
 
-		ValidHandleCheck(handle);
-		maphndletodev(handle, &bd, &ch);
-		get_board_model(bd, s);
-	}
-	catch(const Wobbly &w) {
-		return RunTimeError(w,"vpb_get_model");
-	}
 	return VPB_OK;
 } //}}}
 
@@ -3360,7 +3265,7 @@ int WINAPI vpb_isdn_get_cinfo(int h, VPB_CALL_INFO *cinfo)
 		maphndletodev(h, &b, &ch);
 		VPBREG  *pvpbreg = vpb_c->vpbreg(b);
 		OpenPri *PriDsp  = (OpenPri*)pvpbreg->hostdsp;
-		if( PriDsp->chan_cinfo(ch,(void*)cinfo) != 0 )
+		if( PriDsp->chan_cinfo(ch, cinfo) != 0 )
 			throw Wobbly(VPBAPI_ISDN_CALLINFO_FAILED);
 	      #else
 		(void)h;
