@@ -34,7 +34,6 @@
 #include "mess.h"
 #include "verbose.h"
 
-#include <cassert>
 #include <cstdio>
 #include <cstdlib>
 #include <cstring>
@@ -58,9 +57,7 @@
 #endif
 
 #define	SIZE_TX_Q		800   // size of Tx (down) signal FIFOs
-#define	LENGTH_FRAME	        160   // samples in each processing frame
-
-#define	SIZE_RELAY_BUF	16000	      // size of relay buffer in device driver
+#define	SIZE_RELAY_BUF	        16000 // size of relay buffer in device driver
 
 // default gains
 
@@ -237,7 +234,6 @@ VPBRegister::VPBRegister()
 
 			ioctl(fd, VPB_IOC_PCI_GET_NUM_CARDS, &io);
 			close(fd);
-			mprintf("Found %d OpenLine cards\n", numol);
 
 			//m_conf.cardcount += numol;
 
@@ -247,19 +243,17 @@ VPBRegister::VPBRegister()
 			//    so just crap out here until the user makes a config
 			//    file to tell us what they really have.
 
-			mprintf("You must set up a config file to use\n");
-			mprintf("the OpenLine/OpenLog range of cards.\n");
-			mprintf("The VpbConfigurator utility can assist with that.\n");
-
-			// Not the best message to relay, but near enough for now.
-			throw Wobbly(VPBREG_NO_VPB_DEVICES_DETECTED);
+			throw VpbException("Found %d OpenLine/Log cards, but no config.\n"
+					   "\tRun VpbConfigurator and answer its questions.",
+					   numol);
 		}
 
 		//XXX Do something to detect PRI cards too
 		mprintf("Detected %d cards\n", m_conf.cardcount);
 	}
 	if( ! m_conf.cardcount )
-		throw Wobbly(VPBREG_NO_VPB_DEVICES_DETECTED);
+		throw VpbException("No VPB hardware detected, and none configured.\n"
+				   "\tHave you built and loaded the VPB kernel driver?");
 
 	m_reg.resize( m_conf.cardcount );
 	for(unsigned int i=0; i < m_conf.cardcount; ++i)
@@ -411,10 +405,10 @@ VPBRegister::VPBRegister()
 			break;
 
 		    default:
-			assert(0);
-			break;
+			throw VpbException("Invalid card type %d for card #%u",
+								loc_model, i);
 		}
-		preg.lsf               = LENGTH_FRAME;
+		preg.lsf               = V4PCI_LSF;
 		preg.defRecordGainDown = defRecordGainDown;
 
 		mprintf("defRecordGain = %f\n", preg.defRecordGain);
