@@ -182,10 +182,12 @@ void settonedet(int chdev, const VPB_DETECT &d)
 
 	// if ident doesn't exist, make new tone
 
-	if( ! exists )
+	if( ! exists ) {
+		if(numtd[chdev] >= VPB_MD)
+			throw VpbException("settonedet: too many detectors");
+
 		index = numtd[chdev]++;
-	if(numtd[chdev] > VPB_MD)
-		throw VpbException("settonedet: too many detectors");
+	}
 
 	// validate tone parameters
 
