@@ -927,10 +927,19 @@ void VTCore::SetHookState( int port, HookState hookstate )
 			m_reg.cardnum, port, hookstate, ret, strerror(ret));
 } //}}}
 
+static uint16_t silence[NBUF];
+
+void VTCore::PadTxFrame( int port )
+{ //{{{
+	size_t remains = m_reg.txdf[port]->HowFull() % NBUF;
+
+	if(remains)
+		m_reg.txdf[port]->Write(silence, NBUF - remains);
+} //}}}
+
 void VTCore::WaitForTxEmpty( int port )
 { //{{{
-	static uint16_t     silence[NBUF];
-	static const int    vtcore_fifo_size = 160 * 8;
+	static const int vtcore_fifo_size = 160 * 8;
 
 	for(;;) {
 		int remains = m_reg.txdf[port]->HowFull();
