@@ -68,7 +68,7 @@ static ERROR errors[] = {
 	//{"PORT_CANT_ALLOCATE_MEMORY",					107},
 
 	{"VPBREG_REG_KEY_ACCESS_FAIL",					200},
-	{"VPBREG_NO_VPB_DEVICES_DETECTED",				201},
+	//{"VPBREG_NO_VPB_DEVICES_DETECTED",				201},
 	{"VPBREG_TOO_MANY_VPBS_ACCESSED",				202},
 	{"VPBREG_REG_KEY_PROPERTY_FAIL",				203},
 	//{"VPBREG_CANT_ALLOCATE_MEMORY",				204},
