@@ -315,13 +315,16 @@ int __init vtcore_init(void)
 	if( vtcore->procfs_root ) {
 		struct proc_dir_entry *p;
 
-		vtcore->procfs_root->owner = THIS_MODULE;
+		set_proc_owner(vtcore->procfs_root, THIS_MODULE);
 		p = create_proc_read_entry("maxboards", 0444,
 					   vtcore->procfs_root,
 					   vt_proc_read_int, (void*)MAX_BOARDS);
-		if( p ) p->owner = THIS_MODULE;
-		else    crit("failed to create procfs maxboards");
-	} else crit("failed to create procfs root");
+		if (p) {
+			set_proc_owner(p, THIS_MODULE);
+		} else
+			crit("failed to create procfs maxboards");
+	} else
+		crit("failed to create procfs root");
 
     exit:
 	mutex_unlock(&vtcore_mutex);
@@ -1124,7 +1127,7 @@ int vt_create_board_proc_const_int(struct vtboard *board, const char *node, long
 	p = create_proc_read_entry(node, 0444, board->procfs_root,
 					 vt_proc_read_int, (void*)val);
 	if( p ){
-		p->owner = board->owner;
+		set_proc_owner(p, board->owner);
 		return RET_OK;
 	}
 	crit("failed to create procfs board%d/%s", board->boardnum, node);
@@ -1324,21 +1327,25 @@ int vt_board_register(struct vtboard *board)
 		if( board->procfs_root ) {
 			struct proc_dir_entry *p;
 
-			board->procfs_root->owner = board->owner;
+			set_proc_owner(board->procfs_root, board->owner);
 
 			p = create_proc_read_entry("name", 0444,
 						   board->procfs_root,
 						   vt_proc_read_string,
 						   (void*)board->name);
-			if( p ) p->owner = board->owner;
-			else    crit("failed to create procfs board%d/maxports", i);
+			if (p) {
+				set_proc_owner(p, board->owner);
+			} else
+				crit("failed to create procfs board%d/maxports", i);
 
 			p = create_proc_read_entry("id", 0444,
 						   board->procfs_root,
 						   vt_proc_read_string,
 						   board->serial);
-			if( p ) p->owner = board->owner;
-			else    crit("failed to create procfs board%d/id", i);
+			if (p) {
+				set_proc_owner(p, board->owner);
+			} else
+				crit("failed to create procfs board%d/id", i);
 
 			vt_create_board_proc_const_int(board,"maxports",
 						       board->maxports);
@@ -1394,7 +1401,7 @@ int vt_board_register(struct vtboard *board)
 				   MKDEV( MAJOR(board->name##_dev),		    \
 					  MINOR(board->name##_dev) + i ),	    \
 				   NULL,					    \
-				   "vtboard%d" #name "%d", board->boardnum, i );    \
+				   "vt!board%d!" #name "%d", board->boardnum, i );    \
 		if( IS_ERR(d) ) {						    \
 			portcrit(board,i,"FAILED to register " #name " device (%ld)", \
 				 PTR_ERR(d));					    \
@@ -1412,15 +1419,17 @@ int vt_board_register(struct vtboard *board)
 				struct proc_dir_entry *p;
 				int    mode = 0;
 
-				chan->procfs_root->owner = board->owner;
+				set_proc_owner(chan->procfs_root, board->owner);
 
 				p = create_proc_read_entry("type", 0444,
 							   chan->procfs_root,
 							   vt_proc_read_string,
 						    (void*)porttype(chan->porttype));
-				if( p ) p->owner = board->owner;
-				else    crit("failed to create board%d/port%d/type",
-					     board->boardnum, i);
+				if (p) {
+					set_proc_owner(p, board->owner);
+				} else
+					crit("failed to create board%d/port%d/type",
+								board->boardnum, i);
 
 				mode  = board->set_country ? 0200 : 0;
 				mode |= chan->country ? 0444 : 0;
@@ -1428,13 +1437,14 @@ int vt_board_register(struct vtboard *board)
 				    p = create_proc_entry("country",
 							  board->set_country ? 0644 : 0444,
 							  chan->procfs_root);
-				    if( p ){
-					    p->owner      = board->owner;
+				    if (p) {
+					    set_proc_owner(p, board->owner);
 					    p->read_proc  = proc_read_country;
 					    p->write_proc = proc_write_country;
 					    p->data = (void*)(((long)board->boardnum << 16) | i);
-				    } else  crit("failed to create board%d/port%d/country",
-						 board->boardnum, i);
+				    } else
+					    crit("failed to create board%d/port%d/country",
+									board->boardnum, i);
 				} //}}}
 
 
@@ -1443,8 +1453,8 @@ int vt_board_register(struct vtboard *board)
 			if( mode ){				    		    \
 			    p = create_proc_entry(#attrib, mode,    		    \
 						  chan->procfs_root);	    	    \
-			    if( p ){						    \
-				    p->owner      = board->owner;		    \
+			    if (p) {						    \
+				    set_proc_owner(p, board->owner);		    \
 				    p->read_proc  = proc_read_##attrib;		    \
 				    p->write_proc = proc_write_##attrib;	    \
 				    p->data = (void*)(((long)board->boardnum << 16) | i); \
@@ -1468,9 +1478,12 @@ int vt_board_register(struct vtboard *board)
 									   chan->procfs_root,
 									   proc_read_linevolt,
 						    (void*)(((long)board->boardnum << 16) | i));
-						if( p ) p->owner = board->owner;
-						else    crit("failed to create board%d/port%d/linevolt",
-							     board->boardnum, i);
+						if (p) {
+							set_proc_owner(p, board->owner);
+						} else
+							crit("failed to create board%d/"
+									"port%d/linevolt",
+									board->boardnum, i);
 					}
 					break;
 
@@ -1583,8 +1596,10 @@ void vt_echo_register(struct vtecho *ec)
 								  vtcore->procfs_root,
 								  vt_proc_read_string,
 								  ec->desc);
-		if( p ) p->owner = THIS_MODULE; //XXX ec->owner?
-		else    crit("failed to create procfs echo_canceller");
+		if (p) {
+			set_proc_owner(p, THIS_MODULE);
+		} else
+			crit("failed to create procfs echo_canceller");
 	}
 	info("echo canceller [%s] registered",ec->desc);
 	mutex_unlock(&vtcore_mutex);
