@@ -126,8 +126,10 @@ void vtecho_fp_open(struct channel *chan)
 								  chan->procfs_root,
 								  proc_read_adaptcount,
 								  sv);
-		if( p ) p->owner = THIS_MODULE;
-		else    crit("failed to create procfs echo_adapt_count");
+		if (p) {
+			set_proc_owner(p, THIS_MODULE);
+		} else
+			crit("failed to create procfs echo_adapt_count");
 	}
 	chan->echocan = sv;
 } //}}}
